package com.bwei.auth.service.impl;


import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.bwei.auth.feign.SystemFeignService;
import com.bwei.auth.service.AuthService;

import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Admin;
import com.bwie.common.domain.request.JwtRequest;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;


@Service
public class AuthServiceImpl implements AuthService {
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    @Autowired
    private SystemFeignService feignService;

    @Override
    public Result tocode(String tel) {
        //判断号码
        Result<Admin> loginSel = feignService.toLoginSel(tel);
        Admin admin = loginSel.getData();
        if (admin == null) {
            return Result.error("号码未注册");
        }
        //创建验证码
        String code = RandomUtil.randomNumbers(6);
        redisTemplate.opsForValue().set(tel,code);
        //反响
        JwtRequest jwtRequest = new JwtRequest();
        jwtRequest.setTel(tel);
        jwtRequest.setCode(code);
        return Result.success(jwtRequest);
    }

    @Override
    public Result login(JwtRequest jwtRequest) {
        //判断非空
        if (StringUtils.isAllBlank(jwtRequest.getTel(),jwtRequest.getCode())){
            return Result.error("号码或验证码不能为空");
        }
        //判断号码
        Result<Admin> loginSel = feignService.toLoginSel(jwtRequest.getTel());
        Admin admin = loginSel.getData();
        if (admin == null) {
            return Result.error("");
        }
        //判断验证码
        String code = redisTemplate.opsForValue().get(jwtRequest.getTel());
        if (!jwtRequest.getCode().equals(code)){
            return Result.error("验证码错误");
        }
        //令牌
        String userKey = UUID.randomUUID().toString().replaceAll("-", "");
        Map<String,Object> claims = new HashMap<>();
        claims.put(JwtConstants.USER_KEY,userKey);
        String token = JwtUtils.createToken(claims);
        //存入redis
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userKey, JSONObject.toJSONString(admin));
        //反响
        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setEndTime("30MIN");
        return Result.success(jwtResponse);
    }

    @Override
    public Admin info() {
        //token
        String token = request.getHeader(TokenConstants.TOKEN);
        //userKey
        String userKey = JwtUtils.getUserKey(token);
        //redis
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        //反序列化
        return JSONObject.parseObject(s,Admin.class);
    }
}
