package com.bwie.gateway.filters;

import com.bwie.common.constants.TokenConstants;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.GatewayApp;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.origin.Origin;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.concurrent.TimeUnit;

@Component
public class AuthFiter implements GlobalFilter, Ordered {
    @Autowired
    private IgnoreWhiteConfig ignoreWhiteConfig;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;


    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        //获取白名单
        List<String> whites = ignoreWhiteConfig.getWhites();
        //获取请求的URI
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        if (StringUtils.matches(path,whites)){
            return chain.filter(exchange);
        }
        //token非空
        String token = request.getHeaders().getFirst(TokenConstants.TOKEN);
        if (StringUtils.isEmpty(token)){
            return GatewayUtils.errorResponse(exchange,"token不能为空",HttpStatus.UNAUTHORIZED);
        }
        //token合法性 为假抛出异常
        try {
            JwtUtils.parseToken(token);
        } catch (Exception e) {
            return GatewayUtils.errorResponse(exchange,"token不合法！",HttpStatus.UNAUTHORIZED);
        }
        //token有效
        //获取admin
        String userKey = JwtUtils.getUserKey(token);
        Boolean aBoolean = redisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        if (userKey==null |!aBoolean){
            return GatewayUtils.errorResponse(exchange,"过期",HttpStatus.UNAUTHORIZED);
        }
        //expire
        //放行
        return chain.filter(exchange);
    }

    /**
     *
     * @return
     */
    @Override
    public int getOrder() {
        return 0;
    }
}
